% modelo bsico neo-keynesiano (cap. 3 de "Monetary policy, inflation and
% business cycle", de J. Gal, Ed. Princeton University Press)
clear;
%parameters
epsilon=6;
sigma=1;
varphi=1;
beta=0.99;
rho=-log(beta);
eta=4;
alpha=1/3;
theta=2/3;
M=epsilon/(epsilon-1);
mu=log(M);
rhoa=0.9;
sigmaa=0.01;
rhov=0.5;
sigmav=0.001;
phipi=1.5;
phiy=0.5/4;
Theta=(1-alpha)/(1-alpha+alpha*epsilon);
lambda=(1-beta*theta)*(1-theta)*Theta/theta;
vyn=-(1-alpha)*(mu-log(1-alpha))/(sigma*(1-alpha)+varphi+alpha);
psiyan=(1+varphi)/(sigma*(1-alpha)+varphi+alpha);
kappa=lambda*(sigma+((varphi+alpha)/(1-alpha)));
MA=[1+phiy/sigma  phipi/sigma;
    -kappa        1];
MB=[1   1/sigma;
    0   beta];
MC=[1/sigma;
    0];
MAT=inv(MA)*MB;
MBT=inv(MA)*MC;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PHI1=-sigma*psiyan*(1-rhoa)*(1-rhoa*beta)/...
   (sigma+phiy+kappa*phipi-rhoa*(sigma*(1+beta*(1-rhoa))+kappa+beta*phiy));
PHI2=-(1-rhov*beta)/...
   (sigma+phiy+kappa*phipi-rhov*(sigma*(1+beta*(1-rhov))+kappa+beta*phiy));
PSI1=-sigma*psiyan*(1-rhoa)*kappa/...
   (sigma+phiy+kappa*phipi-rhoa*(sigma*(1+beta*(1-rhoa))+kappa+beta*phiy));
PSI2=-kappa/...
   (sigma+phiy+kappa*phipi-rhov*(sigma*(1+beta*(1-rhov))+kappa+beta*phiy));


%     Innovations
nobs=100;
epst=randn(nobs+100,2);
epsat=epst(101:nobs+100,1)*sigmaa*0.0;
    epsat(10)=sigmaa;
epsvt=epst(101:nobs+100,2)*sigmav*0.0;
at=[];a0=0;vt=[];v0=0;
for t=1:nobs
    a1=rhoa*a0+epsat(t);
    v1=rhov*v0+epsvt(t);
    at=[at;a1];
    vt=[vt;v1];
    a0=a1;
    v0=v1;
end
%variables en logaritmos
    ytildet=PHI1*at+PHI2*vt;
    pit=PSI1*at+PSI2*vt;
    ynt=psiyan*at+vyn;
    yt=ytildet+ynt;
    it=rho+phipi*pit+phiy*ytildet+vt;
    Epit=PSI1*rhoa*at+PSI2*rhov*vt;
    rt=it-Epit;
    nt=(1/(1-alpha))*(yt-at);
    %variables nominales
    p0=0;
    pt=[];ptstar=[];
    for t=1:nobs
        p1=pit(t)+p0;
        p1star=(1/(1-theta))*pit(t)+p0;
        pt=[pt;p1];
        ptstar=[ptstar; p1star];
    end
    mt=yt-eta*it+pt;
    wt=pt+varphi*nt-sigma*yt;

% variables en niveles
Yt=exp(yt);
Ynt=exp(ynt);
Ytildet=exp(ytildet);
Nt=exp(nt);
Pt=exp(pt);
Wt=exp(wt);
Mt=exp(mt);
    
figure;
hold on

subplot(4,2,1)
plot([Ytildet]);
title('output-gap')
xlabel('periods')

subplot(4,2,2)
plot([pit]);
title('inflation')
xlabel('periods')

subplot(4,2,3)
plot([Yt]);
title('output')
xlabel('periods')

subplot(4,2,4)
plot([Nt]);
title('employment')
xlabel('periods')

subplot(4,2,5)
plot([it]);
title('nominal interest rate')
xlabel('periods')

subplot(4,2,6)
plot([rt]);
title('real interest rate')
xlabel('periods')

subplot(4,2,7)
plot([Mt]);
title('money')
xlabel('periods')

subplot(4,2,8)
plot([Wt Pt]);
title('nominal wage rate and prices')
xlabel('periods')

hold off
